@@ -11,6 +11,7 @@
 #include "backend_fapi.h"
 #include "emitter.h"
 #include "parser.h"
+#include "ssl_util.h"
 #include "utils.h"
 
 #ifdef HAVE_FAPI
@@ -665,7 +666,7 @@ CK_RV backend_fapi_update_tobject_attrs(token *t, tobject *tobj, attr_list *attr
     memcpy(&newappdata[0], &appdata[0], tobj_start);
     sprintf((char*)&newappdata[tobj_start], "%08x:%s", tobj->id, attrs);
     memcpy(&newappdata[tobj_start + 9 + strlen(attrs) + 1],
-           &appdata[tobj_start + tobj_len],
+           &appdata[tobj_start + tobj_len + 1],
            appdata_len - tobj_start - tobj_len - 1);
     newappdata[newappdata_len - 1] = '\0';
 
@@ -793,7 +794,7 @@ CK_RV backend_fapi_token_unseal_wrapping_key(token *tok, bool user, twist tpin)
     }
 
     twist sealsalt = user ? tok->fapi.userauthsalt : tok->fapi.soauthsalt;
-    twist sealobjauth = utils_hash_pass(tpin, sealsalt);
+    twist sealobjauth = ssl_util_hash_pass(tpin, sealsalt);
     if (!sealobjauth) {
         rv = CKR_HOST_MEMORY;
         goto error;
@@ -889,7 +890,7 @@ CK_RV backend_fapi_token_changeauth(token *tok, bool user, twist toldpin, twist
     }
     rv = CKR_GENERAL_ERROR;
 
-    oldauth = utils_hash_pass(toldpin, user ? tok->fapi.userauthsalt : tok->fapi.soauthsalt);
+    oldauth = ssl_util_hash_pass(toldpin, user ? tok->fapi.userauthsalt : tok->fapi.soauthsalt);
     if (!oldauth) {
         goto out;
     }
@@ -976,7 +977,8 @@ out:
 
 CK_RV backend_fapi_init(void) {
 
-	return CKR_OK;
+	LOGV("FAPI not enabled, failing init");
+	return CKR_FUNCTION_NOT_SUPPORTED;
 }
 
 CK_RV backend_fapi_destroy(void) {
