@@ -14,9 +14,11 @@
 #define TYPE_BYTE_TEMP_SEQ ((CK_BYTE)5)
 
 void *type_calloc(size_t nmemb, size_t size, CK_BYTE type);
-void *type_realloc(void *orig, size_t size, CK_BYTE type);
+void *type_zrealloc(void *ptr, size_t size, CK_BYTE type);
+
 CK_BYTE type_from_ptr(void *ptr, size_t len);
 CK_RV type_mem_dup(void *in, size_t len, void **dup);
 void type_mem_cpy(void *dest, void *in, size_t size);
+const char *type_to_str(CK_BYTE type);
 
 #endif /* SRC_LIB_TYPED_MEMORY_H_ */
