@@ -201,8 +201,28 @@ class PKCS11ECPublicKey(PKCS11PublicKey):
 
         super(PKCS11ECPublicKey, self).__init__(CKK_EC, attrs, auth, tpm_priv, tpm_pub)
 
+    @staticmethod
+    def ecc_gen_mechs_common(tpm):
+        capdata = tpm.getcap('algorithms')
+        y = yaml.safe_load(capdata)
+
+        # TPM's always support these
+        mechs = [
+            CKM_ECDSA,
+            CKM_ECDSA_SHA1,
+            CKM_ECDSA_SHA256
+        ]
+
+        if 'sha384' in y:
+            mechs.append(CKM_ECDSA_SHA384)
+
+        if 'sha512' in y:
+            mechs.append(CKM_ECDSA_SHA512)
+
+        return mechs
+
     def genmechs(self, tpm):
-        pubmech = [ CKM_ECDSA, CKM_ECDSA_SHA1 ]
+        pubmech = PKCS11ECPublicKey.ecc_gen_mechs_common(tpm)
         self.update({CKA_ALLOWED_MECHANISMS: pubmech})
 
 class PKCS11PrivateKey(PKCS11Key):
@@ -260,19 +280,30 @@ class PKCS11ECPrivateKey(PKCS11PrivateKey):
         super(PKCS11ECPrivateKey, self).__init__(CKK_EC, attrs, auth, tpm_priv, tpm_pub)
 
     def genmechs(self, tpm):
-        privmech = [ CKM_ECDSA, CKM_ECDSA_SHA1 ]
+        privmech = PKCS11ECPublicKey.ecc_gen_mechs_common(tpm)
         self.update({CKA_ALLOWED_MECHANISMS: privmech})
 
 class PKCS11SecretKey(PKCS11Key):
 
     def __init__(self, objtype, attrs, auth=None, tpm_priv=None, tpm_pub=None):
 
+        # If a caller doesn't specify the attribute it matters what kind of key it is. HMAC
+        # keys don't decrypt/encrypt they verify/sign, where as AES keys do the former.
+        def _get_encdec(_a, value, defvalue):
+            if _a in attrs:
+                return attrs[_a]
+
+            if objtype in [ CKK_SHA_1_HMAC, CKK_SHA256_HMAC, CKK_SHA384_HMAC, CKK_SHA512_HMAC ]:
+                return value;
+
+            return defvalue
+
         add = {
             CKA_SENSITIVE: True,
-            CKA_ENCRYPT: attrs[CKA_ENCRYPT],
-            CKA_DECRYPT: attrs[CKA_DECRYPT],
-            CKA_SIGN: attrs[CKA_SIGN] if CKA_SIGN in attrs else attrs[CKA_ENCRYPT],
-            CKA_VERIFY: attrs[CKA_DECRYPT] if CKA_DECRYPT in attrs else attrs[CKA_DECRYPT],
+            CKA_ENCRYPT: _get_encdec(CKA_ENCRYPT, False, True),
+            CKA_DECRYPT: _get_encdec(CKA_DECRYPT, False, True),
+            CKA_SIGN: _get_encdec(CKA_SIGN, True, False),
+            CKA_VERIFY: _get_encdec(CKA_VERIFY, True, False),
             CKA_WRAP: False,
             CKA_UNWRAP: False,
             CKA_EXTRACTABLE: attrs[CKA_EXTRACTABLE],
@@ -304,6 +335,7 @@ class PKCS11AESKey(PKCS11SecretKey):
         mechs = []
         if 'cbc' in y:
             mechs.append(CKM_AES_CBC)
+            mechs.append(CKM_AES_CBC_PAD)
         if 'cfb' in y:
             mechs.append(CKM_AES_CFB128)
         if 'ecb' in y:
@@ -318,7 +350,49 @@ class PKCS11AESKey(PKCS11SecretKey):
 
         self.update({CKA_ALLOWED_MECHANISMS: mechs})
 
-def PKCS11ObjectFactory(public_yaml_data, tpm, auth, init_pubattrs, init_privattrs, tpm_pub, tpm_priv):
+class PKCS11HMACKey(PKCS11SecretKey):
+
+    def __init__(self, ckk_keytype, attrs, auth=None, tpm_priv=None, tpm_pub=None):
+        super(PKCS11HMACKey, self).__init__(ckk_keytype, attrs, auth, tpm_priv, tpm_pub)
+
+    def genmechs(self, tpm):
+
+        ckk_to_ckm = {
+            CKK_SHA_1_HMAC : CKM_SHA_1_HMAC,
+            CKK_SHA256_HMAC : CKM_SHA256_HMAC,
+            CKK_SHA384_HMAC : CKM_SHA384_HMAC,
+            CKK_SHA512_HMAC : CKM_SHA512_HMAC
+        }
+
+        mechs = []
+        key_type = self[CKA_KEY_TYPE]
+        if key_type  == CKK_GENERIC_SECRET:
+            capdata = tpm.getcap('algorithms')
+            y = yaml.safe_load(capdata)
+
+            if 'sha1' in y:
+                mechs.append(CKM_SHA_1_HMAC)
+
+            if 'sha256' in y:
+                mechs.append(CKM_SHA256_HMAC)
+
+            if 'sha384' in y:
+                mechs.append(CKM_SHA384_HMAC)
+
+            if 'sha512' in y:
+                mechs.append(CKM_SHA512_HMAC)
+        elif key_type in ckk_to_ckm:
+            mechs.append(ckk_to_ckm[key_type])
+        else:
+            raise RuntimeError(f'Cannot handle HMAC Key of CKA_KEY_TYPE: {key_type:#x}')
+
+        if len(mechs) == 0:
+            raise RuntimeError("Expected at least one supported mechanism, got none")
+
+        self.update({CKA_ALLOWED_MECHANISMS: mechs})
+
+def PKCS11ObjectFactory(public_yaml_data, tpm, auth, init_pubattrs, init_privattrs,
+        tpm_pub, tpm_priv, override_keylen=None):
 
     objtype = public_yaml_data['type']['value']
     tpmattrs=public_yaml_data['attributes']['value'].split('|')
@@ -414,6 +488,46 @@ def PKCS11ObjectFactory(public_yaml_data, tpm, auth, init_pubattrs, init_privatt
 
         privkey = PKCS11AESKey(privattrs, auth, tpm_pub=tpm_pub, tpm_priv=tpm_priv)
 
+    elif objtype == 'keyedhash':
+
+        alg = public_yaml_data['algorithm']['value']
+        # YAML converts the TPM2_ALG_NULL string of "null" to None, so we need to check against
+        # raw to see if it is TPM2_ALG_NULL. If it is, we just ignore it, as we can use any
+        # supported HASH alg.
+        if alg != 'hmac':
+            if public_yaml_data['algorithm']['raw'] == 0x10:
+                hashalg = 'null'
+            else:
+                raise RuntimeError('Cannot handle algorithm: {}'.format(alg))
+        else:
+            hashalg = public_yaml_data['hash-alg']['value']
+
+        # Their is no PKCS11 metadata for genmechs to reliably determine what the allowed mechanisms are,
+        # since inferring keytype from bytes is a best guess, set up the mechanisms now
+        # since it's not TPM dependent. The allowed mechs are intentionally a list for future expansion.
+        # The keygen mechanism should always go first in the list
+        metadata = {
+            'sha1'   : { 'keybytes' : 20, 'keytype' : CKK_SHA_1_HMAC     },
+            'sha256' : { 'keybytes' : 32, 'keytype' : CKK_SHA256_HMAC    },
+            'sha384' : { 'keybytes' : 48, 'keytype' : CKK_SHA384_HMAC    },
+            'sha512' : { 'keybytes' : 64, 'keytype' : CKK_SHA512_HMAC    },
+            'null'   : { 'keybytes' : override_keylen, 'keytype' : CKK_GENERIC_SECRET }
+        }
+
+        if hashalg not in metadata:
+            raise RuntimeError('Cannot handle hash algorithm: {}'.format(hashalg))
+
+        privattrs[CKA_VALUE_LEN] = metadata[hashalg]['keybytes']
+        privattrs[CKA_SIGN] = 'sign' in tpmattrs
+        privattrs[CKA_VERIFY] = True
+        privattrs[CKA_EXTRACTABLE] = not ('fixedtpm' in tpmattrs and 'fixedparent' in tpmattrs)
+        privattrs[CKA_LOCAL] = 'sensitivedataorigin' in tpmattrs
+        privattrs[CKA_ALWAYS_SENSITIVE] = not privattrs[CKA_EXTRACTABLE]
+        privattrs[CKA_NEVER_EXTRACTABLE] = not privattrs[CKA_EXTRACTABLE]
+        privattrs[CKA_KEY_GEN_MECHANISM] = CKM_GENERIC_SECRET_KEY_GEN
+
+        privkey = PKCS11HMACKey(metadata[hashalg]['keytype'], privattrs, auth, tpm_pub=tpm_pub, tpm_priv=tpm_priv)
+
     else:
         raise RuntimeError("Cannot handle keytype: {}".format(public_yaml_data['type']))
 
